@@ -51,11 +51,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib-object.h>
-#ifdef HAVE_LIBSOUP_GNOME
-#include <libsoup/soup-gnome.h>
-#else
 #include <libsoup/soup.h>
-#endif
 #include <math.h>
 #include <sys/stat.h>
 #include <string.h>
@@ -251,7 +247,7 @@ champlain_network_tile_source_class_init (ChamplainNetworkTileSourceClass *klass
   /**
    * ChamplainNetworkTileSource:proxy-uri:
    *
-   * The proxy uri used to access network
+   * Used to override the default proxy for accessing the network.
    *
    * Since: 0.4
    */
@@ -277,10 +273,8 @@ champlain_network_tile_source_init (ChamplainNetworkTileSource *tile_source)
 
   priv->soup_session = soup_session_async_new_with_options (
         "proxy-uri", NULL,
-#ifdef HAVE_LIBSOUP_GNOME
         SOUP_SESSION_ADD_FEATURE_BY_TYPE, 
-        SOUP_TYPE_PROXY_RESOLVER_GNOME,
-#endif
+        SOUP_TYPE_PROXY_RESOLVER_DEFAULT,
         NULL);
   g_object_set (G_OBJECT (priv->soup_session),
       "user-agent", 
@@ -414,7 +408,7 @@ champlain_network_tile_source_get_proxy_uri (ChamplainNetworkTileSource *tile_so
  * @tile_source: the #ChamplainNetworkTileSource
  * @proxy_uri: the proxy uri used to access network
  *
- * Sets the proxy uri used to access network.
+ * Override the default proxy for accessing the network.
  *
  * Since: 0.6
  */
